@@ -33,6 +33,7 @@ extern "C" {
 #endif
 
 #include <stdarg.h>
+#include <time.h>
 
 #if !defined (_global_h) && !defined (MY_GLOBAL_INCLUDED) /* If not standard header */
 #include <sys/types.h>
@@ -61,6 +62,20 @@ typedef int my_socket;
 #include "ma_list.h"
 #include "mariadb_ctype.h"
 
+
+typedef struct st_ma_const_string
+{
+  const char *str;
+  size_t length;
+} MARIADB_CONST_STRING;
+
+typedef struct st_ma_const_data
+{
+  const unsigned char *data;
+  size_t length;
+} MARIADB_CONST_DATA;
+
+
 #ifndef ST_MA_USED_MEM_DEFINED
 #define ST_MA_USED_MEM_DEFINED
   typedef struct st_ma_used_mem {   /* struct for once_alloc */
@@ -122,11 +137,13 @@ extern unsigned int mariadb_deinitialize_ssl;
   typedef unsigned int MYSQL_FIELD_OFFSET; /* offset to current field */
 
 #define SET_CLIENT_ERROR(a, b, c, d) \
-  { \
+  do { \
     (a)->net.last_errno= (b);\
     strncpy((a)->net.sqlstate, (c), SQLSTATE_LENGTH);\
+    (a)->net.sqlstate[SQLSTATE_LENGTH]= 0;\
     strncpy((a)->net.last_error, (d) ? (d) : ER((b)), MYSQL_ERRMSG_SIZE - 1);\
-  }
+    (a)->net.last_error[MYSQL_ERRMSG_SIZE - 1]= 0;\
+  } while(0)
 
 /* For mysql_async.c */
 #define set_mariadb_error(A,B,C) SET_CLIENT_ERROR((A),(B),(C),0)
@@ -134,11 +151,13 @@ extern const char *SQLSTATE_UNKNOWN;
 #define unknown_sqlstate SQLSTATE_UNKNOWN
 
 #define CLEAR_CLIENT_ERROR(a) \
-  { \
+  do { \
     (a)->net.last_errno= 0;\
     strcpy((a)->net.sqlstate, "00000");\
     (a)->net.last_error[0]= '\0';\
-  }
+    if ((a)->net.extension)\
+      (a)->net.extension->extended_errno= 0;\
+  } while (0)
 
 #define MYSQL_COUNT_ERROR (~(unsigned long long) 0)
 
@@ -205,6 +224,7 @@ extern const char *SQLSTATE_UNKNOWN;
     MYSQL_OPT_MAX_ALLOWED_PACKET,
     MYSQL_OPT_NET_BUFFER_LENGTH,
     MYSQL_OPT_TLS_VERSION,
+    MYSQL_OPT_ZSTD_COMPRESSION_LEVEL,
 
     /* MariaDB specific */
     MYSQL_PROGRESS_CALLBACK=5999,
@@ -234,7 +254,14 @@ extern const char *SQLSTATE_UNKNOWN;
     MARIADB_OPT_MULTI_STATEMENTS,
     MARIADB_OPT_INTERACTIVE,
     MARIADB_OPT_PROXY_HEADER,
-    MARIADB_OPT_IO_WAIT
+    MARIADB_OPT_IO_WAIT,
+    MARIADB_OPT_SKIP_READ_RESPONSE,
+    MARIADB_OPT_RESTRICTED_AUTH,
+    MARIADB_OPT_RPL_REGISTER_REPLICA,
+    MARIADB_OPT_STATUS_CALLBACK,
+    MARIADB_OPT_SERVER_PLUGINS,
+    MARIADB_OPT_BULK_UNIT_RESULTS,
+    MARIADB_OPT_TLS_VERIFICATION_CALLBACK
   };
 
   enum mariadb_value {
@@ -271,7 +298,11 @@ extern const char *SQLSTATE_UNKNOWN;
     MARIADB_CONNECTION_SERVER_STATUS,
     MARIADB_CONNECTION_SERVER_CAPABILITIES,
     MARIADB_CONNECTION_EXTENDED_SERVER_CAPABILITIES,
-    MARIADB_CONNECTION_CLIENT_CAPABILITIES
+    MARIADB_CONNECTION_CLIENT_CAPABILITIES,
+    MARIADB_CONNECTION_BYTES_READ,
+    MARIADB_CONNECTION_BYTES_SENT,
+    MARIADB_TLS_PEER_CERT_INFO,
+    MARIADB_TLS_VERIFY_STATUS
   };
 
   enum mysql_status { MYSQL_STATUS_READY,
@@ -311,7 +342,7 @@ struct st_mysql_options {
     enum mysql_option methods_to_use;
     char *bind_address;
     my_bool secure_auth;
-    my_bool report_data_truncation; 
+    my_bool report_data_truncation;
     /* function pointers for local infile support */
     int (*local_infile_init)(void **, const char *, void *);
     int (*local_infile_read)(void *, char *, unsigned int);
@@ -380,6 +411,20 @@ typedef struct
   void *extension;
 } MYSQL_PARAMETERS;
 
+
+enum mariadb_field_attr_t
+{
+  MARIADB_FIELD_ATTR_DATA_TYPE_NAME= 0,
+  MARIADB_FIELD_ATTR_FORMAT_NAME= 1
+};
+
+#define MARIADB_FIELD_ATTR_LAST MARIADB_FIELD_ATTR_FORMAT_NAME
+
+
+int STDCALL mariadb_field_attr(MARIADB_CONST_STRING *attr,
+                               const MYSQL_FIELD *field,
+                               enum mariadb_field_attr_t type);
+
 #ifndef _mysql_time_h_
 enum enum_mysql_timestamp_type
 {
@@ -400,12 +445,22 @@ typedef struct st_mysql_time
 #define SEC_PART_DIGITS 6
 #define MARIADB_INVALID_SOCKET -1
 
-/* Ansynchronous API constants */
+/* Asynchronous API constants */
 #define MYSQL_WAIT_READ      1
 #define MYSQL_WAIT_WRITE     2
 #define MYSQL_WAIT_EXCEPT    4
 #define MYSQL_WAIT_TIMEOUT   8
 
+#define MARIADB_TLS_VERIFY_OK                  0
+#define MARIADB_TLS_VERIFY_TRUST               1
+#define MARIADB_TLS_VERIFY_HOST                2
+#define MARIADB_TLS_VERIFY_FINGERPRINT         4
+#define MARIADB_TLS_VERIFY_PERIOD              8
+#define MARIADB_TLS_VERIFY_REVOKED            16
+#define MARIADB_TLS_VERIFY_UNKNOWN            32
+#define MARIADB_TLS_VERIFY_ERROR             128  /* last */
+
+
 typedef struct character_set
 {
   unsigned int      number;     /* character set number              */
@@ -432,15 +487,35 @@ typedef struct character_set
   const char *desc;                                     \
   unsigned int version[3];                              \
   const char *license;                                  \
-  void *mariadb_api;                                    \
+  void *mysql_api;                                      \
   int (*init)(char *, size_t, int, va_list);            \
-  int (*deinit)();                                      \
+  int (*deinit)(void);                                  \
   int (*options)(const char *option, const void *);
 struct st_mysql_client_plugin
 {
   MYSQL_CLIENT_PLUGIN_HEADER
 };
 
+enum mariadb_tls_verification {
+  MARIADB_VERIFY_NONE = 0,
+  MARIADB_VERIFY_PIPE,
+  MARIADB_VERIFY_UNIXSOCKET,
+  MARIADB_VERIFY_LOCALHOST,
+  MARIADB_VERIFY_FINGERPRINT,
+  MARIADB_VERIFY_PEER_CERT
+};
+
+typedef struct
+{
+  int version;
+  char *issuer;
+  char *subject;
+  char fingerprint[129];
+  struct tm not_before;
+  struct tm not_after;
+} MARIADB_X509_INFO;
+
+
 struct st_mysql_client_plugin *
 mysql_load_plugin(struct st_mysql *mysql, const char *name, int type,
                   int argc, ...);
@@ -703,7 +778,7 @@ int STDCALL mysql_stmt_send_long_data_cont(my_bool *ret, MYSQL_STMT *stmt,
                                            int status);
 int STDCALL mysql_reset_connection(MYSQL *mysql);
 
-/* API function calls (used by dynmic plugins) */
+/* API function calls (used by dynamic plugins) */
 struct st_mariadb_api {
   unsigned long long (STDCALL *mysql_num_rows)(MYSQL_RES *res);
   unsigned int (STDCALL *mysql_num_fields)(MYSQL_RES *res);
@@ -828,7 +903,7 @@ struct st_mariadb_methods {
 					   const char *db, unsigned int port, const char *unix_socket, unsigned long clientflag);
   void (*db_close)(MYSQL *mysql);
   int (*db_command)(MYSQL *mysql,enum enum_server_command command, const char *arg,
-                    size_t length, my_bool skipp_check, void *opt_arg);
+                    size_t length, my_bool skip_check, void *opt_arg);
   void (*db_skip_result)(MYSQL *mysql);
   int (*db_read_query_result)(MYSQL *mysql);
   MYSQL_DATA *(*db_read_rows)(MYSQL *mysql,MYSQL_FIELD *fields, unsigned int field_count);
@@ -846,12 +921,15 @@ struct st_mariadb_methods {
   void (*set_error)(MYSQL *mysql, unsigned int error_nr, const char *sqlstate, const char *format, ...);
   void (*invalidate_stmts)(MYSQL *mysql, const char *function_name);
   struct st_mariadb_api *api;
+  int (*db_read_execute_response)(MYSQL_STMT *stmt);
+  unsigned char* (*db_execute_generate_request)(MYSQL_STMT *stmt, size_t *request_len, my_bool internal);
 };
 
 /* synonyms/aliases functions */
 #define mysql_reload(mysql) mysql_refresh((mysql),REFRESH_GRANT)
 #define mysql_library_init mysql_server_init
 #define mysql_library_end mysql_server_end
+#define mariadb_connect(hdl, conn_str) mysql_real_connect((hdl),(conn_str), NULL, NULL, NULL, 0, NULL, 0)
 
 /* new api functions */
 
